package com.nokia.tsl_data.service;

import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Map.Entry;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import com.nokia.tsl_data.dao.TslDao;

import lombok.Getter;

/**
 * 计算输出到文件的数据
 */
@Service
public class TslDataService {

    private final TslDao tslDao;

    @Value("${tslTask.compliance.satisfied}")
    private double satisfiedCompliance;
    @Value("${tslTask.compliance.resolution}")
    private double resolutionCompliance;
    @Value("${tslTask.compliance.response}")
    private double responseCompliance;

    @Getter
    private final List<String> areas = new ArrayList<String>() {
        {
            add("保定");
            add("沧州");
            add("承德");
            add("邯郸");
            add("衡水");
            add("廊坊");
            add("秦皇岛");
            add("石家庄");
            add("唐山");
            add("邢台");
            add("雄安");
            add("张家口");
        }
    };

    public TslDataService(TslDao tslDao) {
        this.tslDao = tslDao;
    }

    /**
     * 客户端-投诉问题解决满意度
     * 客户端-投诉问题解决率
     * 客户端-投诉问题响应率
     * 
     * @param day
     * @return
     */
    public List<List<List<Object>>> getSheet4_6Data(String day) {
        String realDay = day.substring(0, 4) + "-" + day.substring(4, 6) + "-" + day.substring(6);
        List<Map<String, Object>> data = tslDao.selectClientRatioForDay(realDay);
        Map<String, Map<String, Object>> dataMap = new HashMap<>();
        for (Map<String, Object> map : data) {
            String key = (String) map.get("businoareaname");
            if (key.equals("雄安新区")) {
                dataMap.put("雄安", map);
            }
            dataMap.put(key, map);
        }
        List<List<List<Object>>> result = new ArrayList<>(3);
        List<List<Object>> list0;
        List<Object> list1;

        // 满意度
        list0 = new ArrayList<>();
        result.add(list0);
        // 地市满意度
        for (String area : areas) {
            list1 = new ArrayList<>(4);
            list0.add(list1);
            // 地市
            list1.add(0, area);
            // 满意率
            list1.add(1, dataMap.get(area).get("complaint_satisfied"));
            // 达标值
            list1.add(2, satisfiedCompliance);
            // 差距
            list1.add(3, ((BigDecimal) list1.get(1)).doubleValue() - (double) list1.get(2));
        }
        // 逆序排序
        list0.sort((o1, o2) -> Double.compare((double) o2.get(3), (double) o1.get(3)));
        // 全省满意度
        list1 = new ArrayList<>(4);
        list0.add(list1);
        // 地市
        list1.add(0, "全省");
        // 满意率
        list1.add(1, dataMap.get("全省").get("complaint_satisfied"));
        // 达标值
        list1.add(2, satisfiedCompliance);
        // 差距
        list1.add(3, ((BigDecimal) list1.get(1)).doubleValue() - (double) list1.get(2));

        // 解决率
        list0 = new ArrayList<>();
        result.add(list0);
        // 地市解决率
        for (String area : areas) {
            list1 = new ArrayList<>(4);
            list0.add(list1);
            // 地市
            list1.add(0, area);
            // 解决率
            list1.add(1, dataMap.get(area).get("complaint_resolution"));
            // 达标值
            list1.add(2, resolutionCompliance);
            // 差距
            list1.add(3, ((BigDecimal) list1.get(1)).doubleValue() - (double) list1.get(2));
        }
        // 逆序排序
        list0.sort((o1, o2) -> Double.compare((double) o2.get(3), (double) o1.get(3)));
        // 全省解决率
        list1 = new ArrayList<>(4);
        list0.add(list1);
        // 地市
        list1.add(0, "全省");
        // 解决率
        list1.add(1, dataMap.get("全省").get("complaint_resolution"));
        // 达标值
        list1.add(2, resolutionCompliance);
        // 差距
        list1.add(3, ((BigDecimal) list1.get(1)).doubleValue() - (double) list1.get(2));

        // 响应率
        list0 = new ArrayList<>();
        result.add(list0);
        for (String area : areas) {
            list1 = new ArrayList<>(4);
            list0.add(list1);
            // 地市
            list1.add(0, area);
            // 解决率
            list1.add(1, dataMap.get(area).get("complaint_response"));
            // 达标值
            list1.add(2, responseCompliance);
            // 差距
            list1.add(3, ((BigDecimal) list1.get(1)).doubleValue() - (double) list1.get(2));
        }
        // 逆序排序
        list0.sort((o1, o2) -> Double.compare((double) o2.get(3), (double) o1.get(3)));
        list1 = new ArrayList<>(4);
        list0.add(list1);
        list1.add(0, "全省");
        // 相应
        list1.add(1, dataMap.get("全省").get("complaint_response"));
        // 达标值
        list1.add(2, responseCompliance);
        // 差距
        list1.add(3, ((BigDecimal) list1.get(1)).doubleValue() - (double) list1.get(2));

        return result;
    }

    /**
     * 超时工单情况
     * 
     * @param day
     * @return
     */
    public List<List<Object>> getSheet3Data1(String day) {
        List<Map<String, Object>> data = tslDao.selectTimeoutTsCountForDay(day);
        Map<String, Map<String, Object>> dayMap = new HashMap<>();
        for (Map<String, Object> map : data) {
            dayMap.put((String) map.get("compl_area_local"), map);
        }
        List<List<Object>> result = new ArrayList<>();
        // 各地市数据
        for (String area : areas) {
            List<Object> list = new ArrayList<>(4);
            result.add(list);
            // 地市
            list.add(0, area);
            // 工单数
            list.add(1, dayMap.get(area).get("total_num"));
            // 超时工单数
            list.add(2, dayMap.get(area).get("timeout_num"));
            // 超时工单占比
            list.add(3, dayMap.get(area).get("timeout_ratio"));
        }
        // 逆序排序
        result.sort((o1, o2) -> {
            return ((BigDecimal) o2.get(3)).compareTo(((BigDecimal) o1.get(3)));
        });
        // 全省数据
        List<Object> list = new ArrayList<>();
        result.add(list);
        // 地市
        list.add(0, "全省");
        // 工单数
        list.add(1, dayMap.get("全省").get("total_num"));
        // 超时工单数
        list.add(2, dayMap.get("全省").get("timeout_num"));
        // 超时工单占比
        list.add(3, dayMap.get("全省").get("timeout_ratio"));
        return result;
    }

    /**
     * 平均处理时长
     * 
     * @param day
     * @return
     */
    public List<List<Object>> getSheet3Data2(String day) {
        Calendar calendar = Calendar.getInstance(Locale.CHINA);
        try {
            calendar.setTime(new SimpleDateFormat("yyyyMMdd").parse(day));
        } catch (ParseException e) {
            e.printStackTrace();
        }
        // 获取上个月的时间
        calendar.add(Calendar.MONTH, -1);
        String oldMonthId = new SimpleDateFormat("yyyyMM").format(calendar.getTime());
        List<Map<String, Object>> oldData = tslDao.selectOldTsDurationForMonth(oldMonthId);
        List<Map<String, Object>> dayData = tslDao.selectTsDurationForDay(day);
        // 转化格式方便读取
        Map<String, Map<String, Object>> dayMap = new HashMap<>();
        Map<String, Map<String, Object>> preMap = new HashMap<>();
        for (Map<String, Object> map : dayData) {
            dayMap.put((String) map.get("compl_area_local"), map);
        }
        for (Map<String, Object> map : oldData) {
            preMap.put((String) map.get("city_name"), map);
        }
        List<List<Object>> result = new ArrayList<>();
        // 各地市数据
        for (String area : areas) {
            List<Object> list = new ArrayList<>();
            result.add(list);
            // 地市
            list.add(0, area);
            // 上月平均处理时长
            list.add(1, preMap.get(area).get("avg_duration"));
            // 当月平均处理时长
            list.add(2, dayMap.get(area).get("avg_duration"));
            // 涨幅
            list.add(3, ((double) list.get(2)) - (double) list.get(1));
        }
        // 逆序排序
        result.sort((o1, o2) -> Double.compare((double) o2.get(3), (double) o1.get(3)));
        return result;
    }

    /**
     * 管理端-重复投诉率
     * 
     * @param day
     * @return
     */
    public List<List<Object>> getSheet2Data(String day) {
        // 从数据库
        List<Map<String, Object>> dayData = tslDao.selectRepeatTsCountForDay(day);
        List<Map<String, Object>> preDayData = dayData;
        if (!day.endsWith("01")) {
            String preDay = String.valueOf(Integer.parseInt(day) - 1);
            preDayData = tslDao.selectRepeatTsCountForDay(preDay);
        }
        // 转化格式方便读取
        Map<String, Map<String, Object>> dayMap = new HashMap<>();
        Map<String, Map<String, Object>> preMap = new HashMap<>();
        for (Map<String, Object> map : dayData) {
            dayMap.put((String) map.get("compl_area_local"), map);
        }
        for (Map<String, Object> map : preDayData) {
            preMap.put((String) map.get("compl_area_local"), map);
        }
        List<List<Object>> result = new ArrayList<>();
        // 各地市数据
        for (String area : areas) {
            List<Object> list = new ArrayList<>(7);
            result.add(list);
            list.add(0, area);
            // 重复投诉工单数
            list.add(1, preMap.get(area).get("repeat_num"));
            list.add(2, dayMap.get(area).get("repeat_num"));
            // 重复投诉率
            list.add(3, preMap.get(area).get("repeat_ratio"));
            list.add(4, dayMap.get(area).get("repeat_ratio"));
            // 增量
            list.add(5, ((BigDecimal) list.get(2)).subtract((BigDecimal) list.get(1)));
            list.add(6, ((BigDecimal) list.get(4)).subtract((BigDecimal) list.get(3)));
        }
        // 排序--逆序
        result.sort((o1, o2) -> {
            return ((BigDecimal) o2.get(6)).compareTo(((BigDecimal) o1.get(6)));
        });
        // 全省数据
        List<Object> list = new ArrayList<>(7);
        result.add(list);
        list.add(0, "全省");
        // 重复投诉工单数
        list.add(1, preMap.get("全省").get("repeat_num"));
        list.add(2, dayMap.get("全省").get("repeat_num"));
        // 重复投诉率
        list.add(3, preMap.get("全省").get("repeat_ratio"));
        list.add(4, dayMap.get("全省").get("repeat_ratio"));
        // 增量
        list.add(5, ((BigDecimal) list.get(2)).subtract((BigDecimal) list.get(1)));
        list.add(6, ((BigDecimal) list.get(4)).subtract((BigDecimal) list.get(3)));
        return result;
    }

    /**
     * 管理端-移网质量类
     * 
     * @param day
     * @return
     */
    public Map<String, List<Object>> getSeet1Data(String day) {
        // 获取当前月
        Calendar calendar = Calendar.getInstance(Locale.CHINA);
        try {
            calendar.setTime(new SimpleDateFormat("yyyyMMdd").parse(day));
        } catch (ParseException e) {
            e.printStackTrace();
        }
        int actualMaximumDayOfMonth = calendar.getActualMaximum(Calendar.DAY_OF_MONTH);
        int dayOfMonth = calendar.get(Calendar.DAY_OF_MONTH);
        // monthId
        String monthId = new SimpleDateFormat("yyyyMM").format(calendar.getTime());

        List<Map<String, Object>> cityTslforMonth = tslDao.selectCityTslForMonth(monthId);
        List<Map<String, Object>> allTslforMonth = tslDao.selectAllTslForMonth(monthId);
        List<Map<String, Object>> cityAllforMonth = tslDao.selectCityAllForMonth(monthId);
        int total = tslDao.selectAllForMonth(monthId);
        List<Map<String, Object>> userCount = getUserCount(monthId);
        List<Map<String, Object>> targetTsRatio = tslDao.selectTargetTsRatioForMonth(monthId);

        Map<String, List<Object>> result = new HashMap<>();
        for (String area : areas) {
            result.put(area, new ArrayList<>());
        }
        result.put("全省", new ArrayList<>());

        // 各地市每日投诉量
        for (Map<String, Object> map : cityTslforMonth) {
            result.get(map.get("compl_area_local")).add(map.get("num"));
        }
        // 全省每日投诉量
        for (Map<String, Object> map : allTslforMonth) {
            result.get("全省").add(map.get("num"));
        }
        // 各地市投诉总量
        for (Map<String, Object> map : cityAllforMonth) {
            result.get(map.get("compl_area_local")).add(map.get("num"));
        }
        // 全省总量
        result.get("全省").add(total);
        // 用户数
        for (Map<String, Object> map : userCount) {
            result.get(map.get("city_name")).add(map.get("user_count"));
        }

        // 万投率和本月预测
        for (Entry<String, List<Object>> entry : result.entrySet()) {
            List<Object> list = entry.getValue();
            int size = list.size();
            // 计算万投率
            double wtl = Double.parseDouble(list.get(size - 2).toString())
                    / Double.parseDouble(list.get(size - 1).toString());
            list.add(wtl);
            // 计算预测万投率
            double ycwtl = wtl * actualMaximumDayOfMonth / dayOfMonth;
            list.add(ycwtl);
        }

        // 期望万投率--从数据库取出来是一个字符串
        for (Map<String, Object> map : targetTsRatio) {
            result.get(map.get("city_name")).add(Double.parseDouble(map.get("target_ts_ratio").toString()));
        }

        // 与目标差距
        for (Entry<String, List<Object>> entry : result.entrySet()) {
            List<Object> list = entry.getValue();
            int size = list.size();
            double cj = ((double) list.get(size - 2)) - ((double) list.get(size - 1));
            list.add(cj);
        }

        return result;
    }

    /**
     * 读取用户数如果当前月份没有数据就读上一个月
     * 
     * 递归方式，最多递归2层
     * 
     * @param monthId
     * @return
     */
    public List<Map<String, Object>> getUserCount(String monthId) {
        return getUserCount(monthId, 2);
    }

    private List<Map<String, Object>> getUserCount(String monthId, int stackLayerCount) {
        List<Map<String, Object>> result = tslDao.selectUserCountForMonth(monthId);
        if (stackLayerCount != 0) {
            if (result == null || result.size() == 0) {
                LocalDate localDate = LocalDate.parse(monthId + "01", DateTimeFormatter.ofPattern("yyyyMMdd"));
                localDate = localDate.plusMonths(-1L);
                String newMonthId = DateTimeFormatter.ofPattern("yyyyMM").format(localDate);
                getUserCount(newMonthId);
            }
        }
        return result;
    }
}
