package com.nokia.tsl_data.service;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.csv.CSVFormat;
import org.apache.commons.csv.CSVParser;
import org.apache.commons.csv.CSVRecord;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import lombok.extern.slf4j.Slf4j;

/**
 * 原始数据入库
 */
@Slf4j
@Service
public class TslWaraHouseService {

    private final JdbcTemplate jdbcTemplate;

    // 河北_CEM移网质量投诉明细 相关
    @Value("${tslTask.source.MOBILE_COMP_SQL}")
    private String MOBILE_COMP_SQL;
    @Value("${tslTask.source.MOBILE_COMP_DIR}")
    private String MOBILE_COMP_DIR;
    @Value("${tslTask.source.MOBILE_COMP_PREFIX}")
    private String MOBILE_COMP_PREFIX;
    @Value("${tslTask.source.MOBILE_COMP_FIELD_NUM}")
    private int MOBILE_COMP_FIELD_NUM;

    // 河北_CEM高品质2日统计 相关 he_d_high_quality
    @Value("${tslTask.source.HIGH_QUALITY_SQL}")
    private String HIGH_QUALITY_SQL;
    @Value("${tslTask.source.HIGH_QUALITY_DIR}")
    private String HIGH_QUALITY_DIR;
    @Value("${tslTask.source.HIGH_QUALITY_PREFIX}")
    private String HIGH_QUALITY_PREFIX;
    @Value("${tslTask.source.HIGH_QUALITY_FIELD_NUM}")
    private int HIGH_QUALITY_FIELD_NUM;

    public TslWaraHouseService(JdbcTemplate jdbcTemplate) {
        this.jdbcTemplate = jdbcTemplate;
    }

    /**
     * 河北_CEM移网质量投诉明细 数据入库
     * report_auto.he_d_mobile_comp
     */
    @Transactional
    public void waraHouseMobileComp(String day) {
        String filePath = MOBILE_COMP_DIR + MOBILE_COMP_PREFIX + day + ".csv";
        try (Reader reader = new InputStreamReader(new FileInputStream(filePath), StandardCharsets.UTF_8)) {
            CSVParser parser = CSVFormat.DEFAULT.builder()
                    .setRecordSeparator("\n")
                    .setQuote(null)
                    .setDelimiter("||")
                    .setSkipHeaderRecord(false)
                    .build().parse(reader);

            List<CSVRecord> records = parser.getRecords();
            List<Object[]> list = new ArrayList<>();
            for (CSVRecord record : records) {
                Object[] ps = new Object[MOBILE_COMP_FIELD_NUM];
                list.add(ps);
                for (int i = 0; i < ps.length; i++) {
                    ps[i] = record.get(i);
                }
            }

            jdbcTemplate.batchUpdate(MOBILE_COMP_SQL, list);
        } catch (IOException e) {
            log.info("csv文件读取错误--{}", filePath);
            e.printStackTrace();
        }
    }

    /**
     * 河北_CEM高品质2日统计 数据入库
     * report_auto.he_d_high_quality
     */
    public void waraHouseHighQuality(String day) {
        String filePath = HIGH_QUALITY_DIR + HIGH_QUALITY_PREFIX + day + ".csv";
        try (Reader reader = new InputStreamReader(new FileInputStream(filePath), StandardCharsets.UTF_8)) {
            CSVParser parser = CSVFormat.DEFAULT.builder()
                    .setRecordSeparator("\n")
                    .setQuote(null)
                    .setDelimiter(",")
                    .setSkipHeaderRecord(false)
                    .build().parse(reader);

            List<CSVRecord> records = parser.getRecords();
            List<Object[]> list = new ArrayList<>();
            for (CSVRecord record : records) {
                Object[] ps = new Object[HIGH_QUALITY_FIELD_NUM];
                list.add(ps);
                for (int i = 0; i < ps.length; i++) {
                    ps[i] = record.get(i);
                }
            }

            jdbcTemplate.batchUpdate(HIGH_QUALITY_SQL, list);
        } catch (IOException e) {
            log.info("csv文件读取错误--{}", filePath);
            e.printStackTrace();
        }
    }
}
